/*---------------------------------------------------------------------------*
 *
 * Project:	fXPlayMidi
 *
 * $RCSfile: file.c,v $
 * $Date: 1997/09/14 01:33:49 $
 * $Author: klmcw $
 * $Revision: 1.7 $
 *
 *---------------------------------------------------------------------------*
 * $Log: file.c,v $
 * Revision 1.7  1997/09/14 01:33:49  klmcw
 * Added state setting for H and V players.
 *
 * Revision 1.6  1997/04/27 02:44:09  klmcw
 * Don't include App->Name in icon title (we know what program it is).
 *
 * Revision 1.5  1997/04/26 08:16:31  klmcw
 * Made title array larger in filePlay_SetState.
 *
 * Revision 1.4  1996/04/21 18:35:47  klmcw
 * Make Window and Icon title include filename during play mode.
 *
 * Revision 1.3  1996/03/02 20:00:23  klmcw
 * Added #includes per -Wall suggestions -- better lint than lint!
 *
 * Revision 1.2  1996/03/02 19:40:08  klmcw
 * Created filePlay_atClose() to be used in fXPLayMidi.c's call to fl_set_atclose().
 *
 * Revision 1.1  1996/02/26 02:23:19  klmcw
 * Initial revision
 *
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
#include <limits.h>
#include <memory.h>
#include <signal.h>
#include <string.h>
#include <sys/wait.h>
#include <unistd.h>

#include "fXPlayMidi.h"
#include "album.h"
#include "child.h"
#include "file.h"
#include "options.h"

/*---------------------------------------------------------------------------*/

static void filePlay_Parent(void);
static void filePlay_SetState(int);

/*---------------------------------------------------------------------------*/

static int Playing = 0, StopRequested = 0;
static pid_t Child = -1;
static FL_OBJECT *btPlay = NULL;

/*---------------------------------------------------------------------------*/
int
fileIsPlaying(void)
{
  return(Playing);
}
/*---------------------------------------------------------------------------*/
void
fileKillChild(void)
{
  /* Send signal to Child's process group */
  kill(-Child, SIGTERM);

  /* Avoid Zombies */
  wait(NULL);
}
/*---------------------------------------------------------------------------*/
void
filePlay(XPMApp_t *App, int Ln)
{
  FL_OBJECT *brp = albumBrp(App);

  if(btPlay == NULL) btPlay = App->Album->btAlbumPlay;

  StopRequested = 0;

  /* Stop action - Kill child if one already exists */
  if(Playing) {
    StopRequested = 1;
    fileKillChild();
    filePlay_SetState(0);
    return;
  }
  /* Play action - set browser to current, fork Child to play */
  fl_deselect_browser(brp);
  fl_select_browser_line(brp, Ln);
  fl_set_browser_topline(brp, Ln);

  switch(Child = fork()) {
  case 0:  childPlay(albumLineGet(App, Ln)); break;
  case -1: fl_show_alert("Error Playing file", "", "", 1); break;
  default: filePlay_Parent(); break;
  }
}
/*---------------------------------------------------------------------------*/
/* Parent's SIGCHLD Handler */
static void
filePlay_SIGCHLD(int sig)
{
  wait(NULL);	/* Prevent Zombies!! */

  filePlay_SetState(0);
  if(optionsRepeatGet() && !StopRequested)
    albumPlay(btPlay->form->u_vdata);
}
/*---------------------------------------------------------------------------*/
int
filePlay_atClose(FL_FORM *fm, void *data)
{
  btExit_cb(btPlay, 0L);

  return(FL_IGNORE);		/* If we get here the selected NO, Don't Quit */
}
/*---------------------------------------------------------------------------*/
static void
filePlay_Parent(void)
{
  signal(SIGCHLD, filePlay_SIGCHLD);

  filePlay_SetState(1);
}
/*---------------------------------------------------------------------------*/
static void
filePlay_SetState(int State)
{
  static const char *Lbls[] = { "Play", "Stop" };
  static const char *pLbls[] = { "@>", "@square" };
  static const char *Shrt[] = { "#P", "#p" };
  static char title[_POSIX_PATH_MAX+1], *titlep;
  XPMApp_t *App = (XPMApp_t *)btPlay->form->u_vdata;
  FL_OBJECT *brp = albumBrp(App);
  XTextProperty Xtitle;

  Playing = State;

  memset(title, '\0', sizeof(title));
  strcpy(title, App->Name);

  if(btPlay != NULL) {
    FL_OBJECT *tbtPlay = NULL;

    tbtPlay = App->HPlayer->btHPlay;
    fl_set_object_label(tbtPlay, pLbls[Playing]);
    fl_redraw_object(tbtPlay);

    tbtPlay = App->VPlayer->btVPlay;
    fl_set_object_label(tbtPlay, pLbls[Playing]);
    fl_redraw_object(tbtPlay);

    fl_set_object_label(btPlay, Lbls[Playing]);
    fl_set_object_shortcut(btPlay, Shrt[Playing], 1);
    fl_redraw_object(btPlay);

    if(!fileIsPlaying()) {
      register int Ln = albumLineNumNext(App);

      fl_deselect_browser(brp);
      fl_select_browser_line(brp, Ln);
      fl_set_browser_topline(brp, Ln);
    } else {
      char *cp, *Linep;
      register int Ln = albumLineNumCurr(App);

      /* Get current file name */
      if((Linep = (char *)fl_get_browser_line(brp, Ln)) == NULL) cp = "UNKNOWN";
      else if((cp = strrchr(Linep, '/')) == NULL)                cp = Linep;
      else {
	cp++;

	/* Don't overwrite Browser Line. */
	sprintf(title, "%s: %s", App->Name, cp);

	/* Don't want the extension */
	if((Linep = strrchr(title, '.')) != NULL)
	  *Linep = '\0';
      }
    }

    fl_set_form_title(App->Album->fmAlbum, title);

    /* Don't want App->Name in icon title. */
    if((titlep = strchr(title, ':'))) titlep += 2;
    else                              titlep = title;

    fl_set_form_title(App->HPlayer->fmHPlayer, titlep);
    fl_set_form_title(App->VPlayer->fmVPlayer, titlep);

    /* Set the Iconic State title -- "Do the X thing" */
    XStringListToTextProperty(&titlep, 1, &Xtitle);
    XSetWMIconName(fl_display, App->Album->fmAlbum->window, &Xtitle);
    XSetWMIconName(fl_display, App->HPlayer->fmHPlayer->window, &Xtitle);
    XSetWMIconName(fl_display, App->VPlayer->fmVPlayer->window, &Xtitle);
  }
}
/*---------------------------------------------------------------------------*/
